import pygame
import data

class surf(object):
    def __init__(self,surface,dp=[0,0]):
        self.surface = surface
        self.dp = dp
    def __getattr__(self,key):
        if key in dir(self.surface):
            return getattr(self.surface,key)
        raise AttributeError(key)
    def paint(self,dest,pos):
        pos = [pos[i]+self.dp[i] for i in range(2)]
        dest.blit(self.surface,pos)

class Sprite:
    def __init__(self):
        self.images = []  #The currently displayed surfaces
        self.pos = [0,0]  #Position to display
        self.size = [0,0] #Size according to image
        self.last_pos_drawn = None  #Only redraw if position changes
        self.dirty = True
        self.behind = None  #Surface of what was behind us before we were
                            #pasted
        self.priority = 0   #higher number = higher layer
        self.remove = False  #If True, remove from world
    def set_remove(self):
        """Mark for removal"""
        self.remove = True
    def do_remove(self):
        """Final remove"""
        self.remove = False
        self.behind = None
        self.dirty = True
        self.last_pos_drawn = None
    def has_changed(self):
        return self.dirty or \
                self.pos != self.last_pos_drawn or self.remove
    def loadimage(self,filename):
        """By default, replaces the image list with [loaded_image]"""
        oi = self.images[:]
        self.images = [surf(data.loadimage(filename))]
        self.size = self.images[0].get_size()
        #Set the image as dirty so renderer redraws it
        if oi!=self.images:
            self.dirty = True

class Man(Sprite):
    def setjob(self,job):
        if job == 'bandit':
            self.speed = 310
        self.job = job
        self.clearcol = clearcol = [246,255,167]
        try:
            self.main_img = surf(data.loadimage('peoples/%s.png'%job,False))
            self.main_img.set_colorkey(clearcol)
            self.choke_img = surf(data.loadimage('peoples/%schoke.png'%job,False))
            self.choke_img.set_colorkey(clearcol)
            self.dead_img = surf(data.loadimage('peoples/%sdead.png'%job,False))
            self.dead_img.set_colorkey(clearcol)
        except:
            pass
        self.num_frames = 0
        try:
            self.set_frame(0,"main")
        except:
            pass
    def aim(self,pos,loc):
        if loc<self.loc:
            self.dir = -1
            return
        if loc>self.loc:
            self.dir = 1
            return
        if pos[0]<self.pos[0]:
            self.dir = -1
            return
        if pos[0]>self.pos[0]:
            self.dir = 1
            return
    def updatetask(self):
        if self.task.tasktype == 'bring_resource':
            #Take resource to dest
            if self.carry == self.task.args[0]:
                b = self.task.args[1]
                #check collision with building
                #complete task
                #aim towards building
                self.aim(b.sprite.pos,b.loc)
            else:
                store = self.task.args[2]
                #check collision with storage
                #get resource
                #aim towards storage
                self.aim(store.sprite.pos,srote.loc)
    def __init__(self,job):
        Sprite.__init__(self)
        self.job = job
        self.loc = 0
        self.carry = None
        self.destination = None
        self.task = None

        self.dir = 1
        self.speed = 300

        self.setjob(job)

        self.carryimages = {}
        self.clearcol = clearcol = [246,255,167]
        if 1:
            img = data.loadimage('othergraphics/metal.png',False)
            img.set_colorkey(clearcol)
            self.carryimages['metal'] = img
            img = data.loadimage('othergraphics/fuel.png',False)
            img.set_colorkey(clearcol)
            self.carryimages['fuel'] = img
            img = data.loadimage('othergraphics/food.png',False)
            img.set_colorkey(clearcol)
            self.carryimages['food'] = img
        else:#except:
            print "DIDNT LOAD"
            pass

        self.animspeed = .1
        self.next_frame = self.animspeed
        self.loop = True

        self.alive = True
        try:
            self.choke_sound = data.loadsound('wuerg.wav')
            self.dead_sound = data.loadsound('man_hit5.wav')
            self.walk_sound = data.loadsound('street_noise.wav')
        except:
            pass

        try:
            self.set_frame(0,"main")
        except:
            pass
    def add_frame(self,dt):
        self.next_frame -= dt
        if self.next_frame <= 0:
            self.next_frame = self.animspeed
            self.frame += 1
            if self.frame>self.num_frames:
                if self.loop:
                    self.frame = 0
                else:
                    return
            self.set_frame(self.frame)
    def set_frame(self,frame,anim=None,loop=None):
        if loop != None:
            self.loop = loop
        if anim:
            anim = getattr(self,anim+"_img")
            if anim!=getattr(self,"anim",None):
                self.dirty = True
                self.anim = anim
                self.num_frames = self.anim.get_width()/30 - 1
        if frame != getattr(self,"last_shown_frame",None):
            self.frame = frame
            self.dirty = True
            self.last_shown_frame = frame
        i = self.anim.subsurface([[frame*30,0],[30,30]])
        if self.dir<0:
            i = pygame.transform.flip(i,-1,0)
        #i.set_colorkey(self.clearcol)
        self.images = [surf(i)]
        self.size = [30,30]

        if self.carry:# and not getattr(self,'lastcarry',None)==self.carry:
            self.dirty = True
            self.lastcarry = self.carry
            i = self.carryimages.get(self.carry,None)
            if i:# and i!=getattr(self,'lastcarrypic',None):
                i = surf(i)
                if not self.alive:
                    i.dp = [0,15]
                self.images.append(i)
    def mouse_click(self):
        if hasattr(self,"click_event"):
            self.click_event()
    def mouse_over(self):
        pass
    def mouse_exit(self):
        pass

class ShipSprite(Sprite):
    def __init__(self,which):
        Sprite.__init__(self)
        self.main_img = data.loadimage('ships/%s.png'%which)
        self.build_img = data.loadimage('ships/%sip.png'%which)
    def set_progress(self,progress):
        print "prog:",progress
        wi,hi = self.main_img.get_size()
        self.size = [wi,hi]
        p1 = 0
        p2 = 0
        which = 1
        togo = 10
        for p in range(int(progress*100)*2):
            if which>0:
                p1 += .01
            else:
                p2 += .01
            togo -= 1
            if togo <= 0:
                togo = 10
                which = -which
        y1 = p1*hi
        y2 = p2*hi
        self.images = [None,None]
        self.images[0] = self.build_img.subsurface([[0,hi-y1],[wi,y1]])
        self.images[0] = surf(self.images[0])
        self.images[0].dp = [0,hi-y1]
        self.images[1] = self.main_img.subsurface([[0,hi-y2],[wi,y2]])
        self.images[1] = surf(self.images[1])
        self.images[1].dp = [0,hi-y2]
        self.dirty = True

class BuildingSprite(Sprite):
    def __init__(self,name=''):
        Sprite.__init__(self)
        self.loadimage('ships/building%s.png'%name)
    def setpos(self,p):
        self.pos = p
        h = self.images[0].get_size()[1]
        y = 480-h-8
        self.pos = [self.pos[0],y]
        self.dirty = True

class Text(Sprite):
    def updatetext(self,text='test',size=None):
        if not size:
            size = 24
        if not hasattr(self,"font") and size:
            self.font = data.loadfont('georgia.ttf',size)
        if hasattr(self,"font"):
            self.text = text
            self.rendertext()
    def rendertext(self):
        wt = surf(self.font.render(self.text,1,[240,240,255]),[4,6])
        bt = surf(self.font.render(self.text,1,[25,30,40]),[6,8])
        while len(self.images)>1:
            del self.images[1]
        self.images.extend([bt,wt])
    def loadimage(self,filename):
        if not hasattr(self,"filename"):
            self.filename = filename
            self.filenameh = filename.replace(".png","")+"h.png"
        Sprite.loadimage(self,filename)
        self.rendertext()

class Button(Text):
    def __init__(self,pos=[0,0],text="",bgimage=None,world=None,size=24):
        Text.__init__(self)
        self.pos = pos[:]
        self.updatetext(text,size)
        if bgimage:
            self.loadimage(bgimage)
        if world:
            world.append(self)
    def mouse_enter(self):
        self.hi = 1
        self.loadimage(self.filenameh)
    def mouse_exit(self):
        self.hi = 0
        self.loadimage(self.filename)
    def mouse_click(self):
        if hasattr(self,"click_event"):
            self.click_event()
