import pygame

from effects import *
from graphics import *
from util import *
from away import *

class ExitState(Exception):
    pass

class NewState(Exception):
    def __init__(self,ns):
        self.state = ns

divx = 4
divy = 4
class State:
    def __init__(self):
        """Initialize a state"""
        self.world = []
        self.backdrop = None
        self.screen = pygame.display.get_surface()
        self.throw = None
        self.clock = pygame.time.Clock()
        self.dt = 0
    def sort_world(self):
        def cmp_sprite(sp1,sp2):
            return cmp(sp1.priority,sp2.priority)
        self.world.sort(cmp_sprite)
        for sprite in self.world:
            self.move(sprite)
    def move(self,sprite,pos=None):
        if pos:
            sprite.pos = pos[:]
    def _beforeloop(self):
        """What to do before we go into a loop"""
        self.updatedrects = []
    def loop(self):
        """Run until we should stop"""
        self._beforeloop()
        run = True
        ret = None
        while run:
            try:
                self.iteration()
            except ExitState:
                run = False
            except NewState,e:
                run = False
                ret = e.state
        img = self.screen.convert()
        fade(img,0.25,mode=-1,screen=self.screen)
        return ret
    def iteration(self):
        """One iteration"""
        if self.throw:
            raise self.throw
        self.dt = self.clock.tick(60)/1000.0
        self._beforerender()
        self._render()
        self._afterrender()
        self._events()
    def _beforerender(self):
        """Do stuff before we render"""
    def _render(self):
        """Update the display"""
        self.updatedrects = []
        changed = 0
        while changed < len(self.world):
            if self.world[changed].has_changed():
                break
            changed += 1
        clear = len(self.world)-1
        while clear >= changed:
            if self.world[clear].behind and \
                            self.world[clear].last_pos_drawn:
                self.screen.blit(self.world[clear].behind,
                        self.world[clear].last_pos_drawn)
                self.updatedrects.append([self.world[clear].last_pos_drawn,
                    self.world[clear].size])
            if self.world[clear].remove:
                self.world[clear].do_remove()
                self.world.remove(self.world[clear])
            clear -= 1
        draw = changed
        while draw < len(self.world):
            self.drawnew(self.world[draw])
            draw += 1
        #pygame.display.update(self.updatedrects)
        pygame.display.flip()
    def _events(self):
        """Low level event management"""
        for e in pygame.event.get():
            if e.type==pygame.QUIT:
                raise ExitState
            else:
                self.event(e)
    def _afterrender(self):
        """Do stuff when rendering is done"""
    def event(self,e):
        """Handle some event"""
    def assign_back(self,sprite,pos,size,surf=None):
        if not surf: surf = self.screen
        sprite.behind = pygame.Surface(sprite.size).convert()
        sprite.behind.blit(surf,[0,0],[sprite.pos,sprite.size])
    def drawnew(self,sprite,surf=None):
        """Clear old spot of sprite, update sprite position
        and draw sprite in new position"""
        if not surf:
            surf = self.screen
        self.updatedrects.append([sprite.pos,sprite.size])
        self.assign_back(sprite,sprite.pos,sprite.size,surf)
        sprite.last_pos_drawn = sprite.pos[:]
        sprite.dirty = False
        for img in sprite.images:
            img.paint(surf,sprite.pos)

class MouseState(State):
    """Any state that uses a mouse"""
    def __init__(self):
        State.__init__(self)
        self.cursor = Sprite()
        self.cursor.loadimage('gui/cursor.png')
        self.cursor.priority = 500
        self.cursor.over = []
        self.world.append(self.cursor)
    def event(self,e):
        State.event(self,e)
        if e.type==pygame.MOUSEMOTION:
            self.move(self.cursor,e.pos[:])
        if e.type==pygame.MOUSEBUTTONUP:
            for b in self.cursor.over:
                if hasattr(b,"mouse_click"):
                    b.mouse_click()
    def _beforerender(self):
        nothit = []
        hit = []
        for sprite in self.world:
            if sprite == self.cursor: continue
            if point_in_rect(self.cursor.pos,sprite.pos,sprite.size):
                hit.append(sprite)
            else:
                nothit.append(sprite)
        for sprite in hit:
            if sprite not in self.cursor.over:
                self.cursor.over.append(sprite)
                if hasattr(sprite,"mouse_enter"):
                    sprite.mouse_enter()
        for sprite in nothit:
            if sprite in self.cursor.over:
                self.cursor.over.remove(sprite)
                if hasattr(sprite,"mouse_exit"):
                    sprite.mouse_exit()
        State._beforerender(self)

class GameState(MouseState):
    def quit(self):
        self.throw = ExitState()
    def start(self):
        self.throw = NewState(InGameState)

from logic import *
import random
class InGameState(GameState):
    def __init__(self):
        GameState.__init__(self)

        skipper = Handler()
        for i in range(6):
            i2 = i + 1
            fade(data.loadimage('design/tutorial%s.png'%i2),1)
            try:
                pause(60,skipper)
            except:
                pass
            fade(data.loadimage('design/tutorial%s.png'%i2),1,mode=-1)


        pygame.mixer.music.load(data.filepath('music/song2.ogg'))
        pygame.mixer.music.play(0)
        
        self.schedule = timer_queue()
        self.schedule.add_recurring(self.update_hq,1,2)
        self.schedule.add_recurring(self.update_gate,1,0.25)
        self.schedule.add_recurring(self.makebandit,10,2.5)
        
        self.date = 0
        self.sec_per_unit = 5
        self.peoples = []
        
        self.rightwall = 2
        self.leftwall = -1

        self.academy = Academy(loc=-1)
        self.academy.sprite = BuildingSprite('acad')
        self.academy.sprite.setpos([350,0])
        self.hq = HQ(loc=0)
        self.hq.sprite = BuildingSprite('head')
        self.hq.sprite.setpos([35,0])
        self.storage = Storage(loc=1)
        self.storage.sprite = BuildingSprite('stor')
        self.storage.sprite.setpos([300,0])
        self.gate = Gate(loc=self.rightwall)

        self.oxygen = 100000

        self.taskmaster = TaskMaster()
        

        possible_people = ['bandit','carrier','civilian',
                'explorer','guard','medic','scientist']
        possible_people = ['civilian']
        #possible_people = ['carrier']
        for i in range(30):#range(35):
            people = Man(random.choice(possible_people))
            #people.carry = 'fuel'
            people.click_event = self.killman(people)
            self.hq.peoples.append(people)
        
        self.ship = Ship(0)
        self.ship.sprite = ShipSprite('ship1')
        self.ship.add_metal(0)
        self.ship.sprite.pos = [100,0]
        self.shipbuilder = Builder(0,self.ship,self.taskmaster)
        self.shipbuilder.sprite = self.ship.sprite

        self.farm = Farm(2)
        self.farm.sprite = BuildingSprite('meat')
        self.farm.sprite.setpos([200,0])
        
        self.next_drop = {"metal":[self.shipbuilder],
                "fuel":[self.shipbuilder,self.farm],
                "food":[self.storage]}

        self.builders = [self.shipbuilder]
        self.setloc(0)

        self.team = Team()
        self.teams = [] #teams leaving

        self.can_launch = 0
        self.do_launch = 0
    def iteration(self):
        #If this is our first run, reset the clock
        #so that the fade time doesn't count in dt
        rc = getattr(self,'_resetclock',0)
        if not rc:
            oldscr = self.screen
            self.screen = pygame.Surface([640,480])
        if rc<2:
            self.clock = pygame.time.Clock()
        if rc==1:
            self._resetclock = 2
        GameState.iteration(self)
        if not rc:
            fade(self.screen,1,screen=oldscr)
            self.screen = oldscr
            self._resetclock = 1
    def killman(self,man=None):
        def f(man=man):
            if not man.alive: return
            if getattr(man,"team",None): return
            man.set_frame(0,'dead',False)
            man.alive = False
            playsound(man.dead_sound,True,1)
            #Put bodies to back layer so they don't have to be updated
            man.priority = -100
            if man.carry:
                setattr(self.storage,
                        man.carry+"_store",
                        getattr(self.storage,man.carry+"_store")+1)
                man.carry = None
            if man in self.world:
                self.world.remove(man)
                self.world.insert(self.peoples_entrance,man)
                self.peoples_entrance += 1   #Make new people come out in front
            #of the dead body
        return f
    def update_hq(self):
        """Should be run every second, to spit out one
        people per second"""
        if self.hq.peoples:
            people = self.hq.peoples.pop(-1)
            people.dir = -1
            people.pos = [100,480-8-30]
            self.peoples.append(people)
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
    def update_gate(self):
        """Should be run every second, to spit out one
        people per second"""
        if self.gate.peoples:
            people = self.gate.peoples.pop(-1)
            people.last_carry = None
            print "from gate carry:",people.carry
            people.dirty = True
            people.dir = -1
            people.set_frame(0,"main")
            people.pos = [640,480-8-30]
            people.speed = 30
            self.schedule.insert(self.speedup(people),next=10)
            self.peoples.append(people)
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
            print people.carry
    def speedup(self,man):
        def f():
            man.speed = 300
        return f
    def makebandit(self):
        guards = 0
        notbandits = []
        for people in self.peoples:
            if people.job is not "bandit" and people.alive and not\
                getattr(people,'team',None):
                notbandits.append(people)
            if people.job == "guard" and people.alive:
                guards +=1
        if not notbandits: return
        if random.randint(1,100)>92:
            print "MAKEBANDIT"
            makethisguybandit = random.choice(notbandits)
            makethisguybandit.setjob("bandit")
            if makethisguybandit.carry:
                c = makethisguybandit.carry+"_store"
                setattr(self.storage,c,getattr(self.storage,c)+1)
                makethisguybandit.carry = None
    def infront(self,people):
        """Returns someone in front of the people"""
        for target in self.peoples:
            if not target.alive:
                continue
            if target == people:
                continue
            if target.loc != people.loc:
                continue
            if people.dir>0:
                if target.pos[0]>people.pos[0]:
                    if target.pos[0]<60+people.pos[0]:
                        return target
            if people.dir<0:
                if target.pos[0]<people.pos[0]:
                    if target.pos[0]>people.pos[0]-60:
                        return target
    def _beforerender(self):
        if self.ship.p>=1:
            if not self.can_launch:
                self.setloc(0)
        GameState._beforerender(self)
        self.date += self.dt
        self.schedule.process(self.dt)
        self.farm.update()
        for building in [self.hq]:
            self.oxygen -= len(building.peoples)*self.dt
        currentlocmen = [x for x in self.world if isinstance(x,Man)]
        if self.do_launch:
            for x in self.peoples[:]:
                if not x.alive:
                    self.peoples.remove(x)
            if not self.peoples:
                if not hasattr(self.ship,"fire"):
                    self.ship.fire = True
                    fire = surf(data.loadimage('ships/flame.png'))
                    fire.dp = [-20,440]
                    self.ship.sprite.images.append(fire)
                    self.ship.sprite.dirty = True
                    self.ship.sprite.size = [640,1000]
                    snd = data.loadsound('rocket1.wav')
                    snd.play()
                self.ship.sprite.pos[1]-=200*self.dt
            if self.ship.sprite.pos[1]<-640:
                print "You won"
                raise ExitState()
        for man in self.peoples:
            man.add_frame(self.dt)
            if man.alive:
                self.oxygen -= self.dt
                if self.date - getattr(man,'last_march',0) > 0.5:
                    playsound(man.walk_sound,False,
                            random.randint(1,3)/10.0)
                    man.last_march = self.date
                t = getattr(man,"team",None)
                if t:
                    if man.loc == self.rightwall:
                        if man.pos[0]>man.spot:
                            if man not in man.team.ready:
                                man.team.ready.append(man)
                            print len(man.team.ready),len(man.team.waiting)
                            if len(man.team.ready)==len(man.team.waiting):
                                self.team_leaves(man.team)
                            continue
                    man.dir = 1
                if self.do_launch:
                    p = self.ship.sprite.pos[:]
                    p[0]+=100
                    man.aim(p,self.ship.loc)
                np = [man.pos[0]+self.dt*man.speed*man.dir,man.pos[1]]
                if self.do_launch:
                    if man.loc == self.ship.loc:
                        if man.pos[0]>=self.ship.sprite.pos[0]+20 and\
                                man.pos[0]<=self.ship.sprite.pos[0]+150:
                            self.peoples.remove(man)
                            man.set_remove()
                if np[0]>640:
                    if man.loc == self.rightwall:
                        man.dir = -1
                    else:
                        man.loc += 1
                        np[0] = 0
                        if man in self.world:
                            man.set_remove()
                        if man.loc == self.loc and man not in self.world:
                            self.world.insert(self.peoples_entrance,man)
                if np[0]<-30:
                    if man.loc == self.leftwall:
                        man.dir = 1
                    else:
                        man.loc -= 1
                        np[0] = 640
                        if man in self.world:
                            man.set_remove()
                        if man.loc == self.loc and man not in self.world:
                            self.world.insert(self.peoples_entrance,man)
                ndlist = self.next_drop.get(man.carry,[])
                if ndlist:
                    nd = ndlist[0]
                    if np[0]>nd.sprite.pos[0] and man.loc==nd.loc:
                        if np[0]<nd.sprite.pos[0]+nd.sprite.size[0]:
                            c = self.next_drop[man.carry].pop(0)
                            self.next_drop[man.carry].append(c)
                            setattr(nd,man.carry+"_store",
                                    getattr(nd,man.carry+"_store")+1)
                            man.carry = None
                if np[0]>self.academy.sprite.pos[0] and np[0]<self.academy.sprite.pos[0]+100:
                    if man.loc == self.academy.loc:
                        if man.job == 'civilian' and self.academy.queue:
                            job = self.academy.queue.pop(0)
                            man.setjob(job)
                if np[0]>self.storage.sprite.pos[0] and np[0]<self.storage.sprite.pos[0]+100:
                    if man.loc == self.storage.loc:
                        if man.job == "carrier":
                            if not man.carry:
                                r = self.storage.getresource()
                                if r:
                                    man.carry = r
                if np[0]>self.farm.sprite.pos[0] and \
                        np[0]<self.farm.sprite.pos[0]+self.farm.sprite.size[0]:
                    if man.loc == self.farm.loc:
                        if man.job == "carrier":
                            if not man.carry:
                                r = self.farm.getresource()
                                if r:
                                    man.carry = r
                if man.job == "bandit":
                    target = self.infront(man)
                    if target and target.job!='bandit':
                        if target.job == 'guard':
                            chance = 1
                        else:
                            chance = 10
                        if random.randint(1,100)<chance:
                            self.killman(target)()
                        if target.job == 'guard':
                            if random.randint(1,100)<1:
                                self.killman(man)()
                if man.job == "guard":
                    target = self.infront(man)
                    if target and target.job=='bandit':
                        if random.randint(1,100)<75:
                            self.killman(target)()
                        else:
                            if random.randint(1,100)<10:
                                self.killman(man)()
                self.move(man,np)
        if self.oxygen <= 0:
            self.oxygen = 0
            if random.randint(0,1000)>950:
                alivemen = [x for x in self.peoples \
                    if x.alive]
                if alivemen:
                    people = random.choice(alivemen)
                    people.alive = False
                    people.set_frame(0,'choke',False)
                    if people.loc == self.loc:
                        playsound(people.choke_sound,True,1)
        self.oxygen_g.updatetext('Oxygen=%d'%self.oxygen)
        self.date_g.updatetext('Date:%d'%(self.sec_per_unit*self.date))
        numcivil = len([x for x in self.peoples if x.job=='civilian']+\
            [x for x in self.hq.peoples if x.job=='civilian'])
        if self.car:
            self.car.updatetext('train carriers %s/%s'%\
                    (len([x for x in self.academy.queue if x=='carrier']),
                    numcivil))
            self.grd.updatetext('train guards %s/%s'%\
                    (len([x for x in self.academy.queue if x=='guard']),
                    numcivil))
            self.acex.updatetext('train explor %s/%s'%\
                    (len([x for x in self.academy.queue if x=='explorer']),
                    numcivil))
            self.acme.updatetext('train medics %s/%s'%\
                    (len([x for x in self.academy.queue if x=='medic']),
                    numcivil))
        if getattr(self,"awcar",None):
            self.awcar.updatetext('send carriers(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='carrier']))
        if getattr(self,"awgu",None):
            self.awgu.updatetext('send guards(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='guard']))
        if getattr(self,"awmed",None):
            self.awmed.updatetext('send medics(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='medic']))
        if getattr(self,"awex",None):
            self.awex.updatetext('send explorer(%s)'%\
                    len([x for x in self.team.waiting if\
                        x.job=='explorer']))
        if getattr(self,"spendfood",None):
            self.spendfood.updatetext('pay (1/%s food) for 10 people'%\
                    self.storage.food_store)
        if hasattr(self,"food_g"):
            self.food_g.updatetext('food:%s'%self.storage.food_store)
        for builder in self.builders:
            builder.update(self.dt)
    def setloc(self,loc):
        self.cursor.over = []
        self.loc = loc
        self.screen.fill([0,0,0])
        #pygame.display.flip()
        for sprite in self.world:
            sprite.do_remove()
        self.world = []
        self.peoples_entrance = 0
        #add background
        self.sky = Sprite()
        self.sky.loadimage('backdrops/sky2.png')
        self.world.append(self.sky)
        self.peoples_entrance += 1
        dome = Sprite()
        dome.loadimage('backdrops/dome.png')
        self.world.append(dome)
        self.peoples_entrance += 1

        #add ship, buildings
        if loc==0:
            self.world.append(self.ship.sprite)
            self.world.append(self.hq.sprite)
            self.peoples_entrance += 2
        if loc==-1:
            self.world.append(self.academy.sprite)
        if loc==1:
            self.world.append(self.storage.sprite)
        if loc==2:
            self.world.append(self.farm.sprite)
        if loc==self.rightwall:
            spr = Sprite()
            spr.loadimage('ships/sign.png')
            spr.pos = [580,480-spr.size[1]-8]
            self.world.append(spr)
        #remember spot
        self.peoples_entrance = len(self.world)
        for people in self.peoples:
            if self.loc == people.loc:
                self.world.insert(self.peoples_entrance,people)
        #go through people and add ones on this screen
        
        #add foreground
        #add menu
        ad = Button([0,40],"Academy","gui/button_1.png",self.world,14)
        ad.click_event = self.set_locm1
        hq = Button([150,40],"Headquarters","gui/button_1.png",self.world,14)
        hq.click_event = self.set_loc0
        ss = Button([300, 40],"Storage","gui/button_1.png",self.world,14)
        ss.click_event = self.set_loc1
        fr = Button([450,40],"Farm","gui/button_1.png",self.world,14)
        fr.click_event = self.set_loc2
        #location specific buttons
        self.car=None
        if loc == -1:
            car = Button([0,80],"","gui/button_1.png",self.world,14)
            car.click_event = self.academy.add_carrier
            self.car=car
            grd = Button([0,120],"","gui/button_1.png",self.world,14)
            grd.click_event = self.academy.add_guard
            self.grd=grd
            self.acex = Button([0,160],"","gui/button_1.png",self.world,14)
            self.acex.click_event = self.academy.add_explorer
            self.acme = Button([0,200],"","gui/button_1.png",self.world,14)
            self.acme.click_event = self.academy.add_medic
        if loc == 0:
            self.away = Button([400,120],"away","gui/button_1.png",self.world)
            self.away.click_event = self.goaway
            self.clr = Button([200,120],"Clear team","gui/button_1.png",
                    self.world,14)
            self.clr.click_event = self.clearteam
            self.awcar = Button([0,80],"send carrier","gui/button_1.png",
                    self.world,14)
            self.awcar.click_event = self.team_add_carrier
            self.awgu = Button([200,80],"send guard","gui/button_1.png",
                    self.world,14)
            self.awgu.click_event = self.team_add_guard
            self.awmed = Button([400,80],"send medic","gui/button_1.png",
                    self.world,14)
            self.awmed.click_event = self.team_add_medic
            self.awex = Button([0,120],"send explorer","gui/button_1.png",
                    self.world,14)
            self.awex.click_event = self.team_add_explorer
        if loc == self.storage.loc:
            self.spendfood = Button([0,120],"Add civilian","gui/button_1.png",
                    self.world,14)
            self.spendfood.click_event = self.add_civilian
        if self.ship.p>=1.0:
            self.can_launch = 1
            launch = Button([50,300],"Launch","gui/button_1.png",
                    self.world)
            launch.click_event = self.launch
        #add mouse again
        self.world.append(self.cursor)
        #Add interface
        self.oxygen_g = Text()
        self.oxygen_g.updatetext('O2 = ',14)
        self.oxygen_g.loadimage('gui/button_1.png')
        self.world.append(self.oxygen_g)
        self.date_g = Text()
        self.date_g.pos = [200,0]
        self.date_g.updatetext('date:',14)
        self.date_g.loadimage('gui/button_1.png')
        self.world.append(self.date_g)

        self.food_g = Text()
        self.food_g.pos = [400,0]
        self.food_g.updatetext('food:',14)
        self.food_g.loadimage('gui/button_1.png')
        self.world.append(self.food_g)

    def launch(self):
        if not self.do_launch:
            self.do_launch = 1
            for p in self.hq.peoples:
                self.peoples.append(p)
                self.world.insert(self.peoples_entrance,p)
            self.hq.peoples = []

    def add_civilian(self):
        if self.storage.food_store:
            self.storage.food_store -= 1
            for i in range(10):
                people = Man('civilian')
                people.click_event = self.killman(people)
                self.hq.peoples.append(people)

    def set_loc1(self):
        self.setloc(1)
    
    def set_loc0(self):
        self.setloc(0)
    
    def set_locm1(self):
        self.setloc(-1)

    def set_loc2(self):
        self.setloc(2)
        
    def clearteam(self):
        self.team.teammembers = []
        for p in self.team.waiting:
            #Make sure people can have a task again
            pass
        self.team.waiting = []

    def team_add_carrier(self):
        possible = [x for x in self.peoples if x.job=='carrier']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_guard(self):
        possible = [x for x in self.peoples if x.job=='guard']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_medic(self):
        possible = [x for x in self.peoples if x.job=='medic']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return
    def team_add_explorer(self):
        possible = [x for x in self.peoples if x.job=='explorer']
        for p in possible:
            if p.carry: continue
            if p in self.team.waiting: continue
            self.team.waiting.append(p)
            return


    def goaway(self):
        team = self.team
        lineup = 610
        team.waiting = [x for x in team.waiting if x.alive]
        for people in team.waiting:
            if people in self.peoples:
                team.teammembers.append(people.job)
                team.y = people.pos[1]
                people.team = team
                people.spot = lineup
                lineup-=10
            else:
                print "error?"
        self.teams.append(team)
        self.team = Team()
    
    def team_leaves(self,team):
        for people in team.ready:
            if people in self.peoples:
                self.peoples.remove(people)
                people.set_remove()
        runmission(team,1,random.randint(1,3))
        self.oxygen -= team.triptime*5
        self.schedule.insert(self.comeback(team),next=team.triptime)
        
    def comeback(self,team):
        def f():
            for job in team.teammembers:
                people = Man(job)
                people.loc = self.rightwall
                people.pos[0] = 640
                people.pos[1] = team.y
                next = ['fuel','metal']
                if job == "carrier" and team.resources:
                    while next[0] not in team.resources:
                        pop = next.pop(0)
                        next.append(pop)
                    people.carry = next[0]
                    people.carry = team.resources[0]
                    team.resources.remove(next[0])
                    pop = next.pop(0)
                    next.append(pop)
                self.gate.peoples.append(people)
        return f
                    
